compile_args = {
    "opt": [],
    "fastbuild": [],
    "dbg": ["-race"],
}

build_args = {
    "opt": [],
    "fastbuild": [],
    "dbg": ["-race"],
}

link_args = {
    "opt": [
        "-w",
        "-s",
    ],
    "fastbuild": [
        "-w",
        "-s",
    ],
    "dbg": ["-race"],
}

def go_compile(ctx, pkg, srcs, archive, setupGOPATH=False, extra_archives=[]):
  gotool = ctx.file._go
  goroot = ctx.file._goroot

  archives = []
  recursive_deps = set()
  include_paths = ""
  for dep in ctx.attr.deps:
    include_paths += "-I \"" + dep.go_archive.path + "_gopath\" "
    archives += [dep.go_archive]
    recursive_deps += dep.go_recursive_deps

  for a in extra_archives:
    include_paths += "-I \"" + a.path + "_gopath\" "
  archives += list(extra_archives)
  recursive_deps += archives

  if setupGOPATH:
    symlink = ctx.new_file(ctx.configuration.bin_dir, ctx.label.name + ".a_gopath/" + pkg + ".a")
    symlink_content = ctx.label.name + ".a"
    for component in pkg.split("/"):
      symlink_content = "../" + symlink_content

  if ctx.attr.go_build:
    # Cheat and build the package non-hermetically (usually because there is a cgo dependency)
    # TODO(schroederc): add -a flag to regain some hermeticity
    args = build_args[ctx.var['COMPILATION_MODE']]
    cmd = (
        "export PATH;" +
        "GOROOT=" + goroot.path + " " +
        "GOPATH=\"$PWD/" + ctx.label.package + "\" " +
        gotool.path + " build " + ' '.join(args) + " -o " + archive.path + " " + ctx.attr.package)
    mnemonic = 'GoBuild'
  else:
    args = compile_args[ctx.var['COMPILATION_MODE']]
    cmd = "\n".join([
        "GOROOT=" + goroot.path,
        'if ' + gotool.path + ' tool | grep -q 6g; then',
        '  TOOL=6g',
        'else',
        '  TOOL=compile',
        'fi',
        gotool.path + " tool $TOOL " + ' '.join(args) + " -p " + pkg + " -complete -pack -o " + archive.path + " " +
        include_paths + " " + cmd_helper.join_paths(" ", set(srcs)),
    ])
    mnemonic = 'GoCompile'

  outs = [archive]
  cmd = "set -e;" + cmd + "\n"
  if setupGOPATH:
    cmd += "ln -sf " + symlink_content + " " + symlink.path + ";"
    outs += [symlink]

  ctx.action(
      inputs = srcs + archives + [gotool, goroot],
      outputs = outs,
      mnemonic = mnemonic,
      command = cmd,
      use_default_shell_env = True)

  return recursive_deps

def link_binary(ctx, binary, archive, recursive_deps):
  gotool = ctx.file._go
  goroot = ctx.file._goroot

  include_paths = ""
  for a in recursive_deps + [archive]:
    include_paths += "-L \"" + a.path + "_gopath\" "

  args = link_args[ctx.var['COMPILATION_MODE']]
  cmd = "\n".join([
      'set -e',
      'export PATH',
      'if ' + gotool.path + ' tool | grep -q 6l; then',
      '  TOOL=6l',
      'else',
      '  TOOL=link',
      'fi',
      gotool.path + ' tool $TOOL ' + ' '.join(args)
      + ' ' + include_paths + ' -o ' + binary.path + ' '
      + ' ' + archive.path + ';',
  ])
  ctx.action(
      inputs = list(recursive_deps) + [archive, gotool, goroot],
      outputs = [binary],
      mnemonic = 'GoLink',
      command = cmd,
      use_default_shell_env = True)

def go_library_impl(ctx):
  archive = ctx.outputs.archive
  pkg = ctx.attr.go_package_prefix + ctx.label.package
  if ctx.attr.package != "":
    pkg = ctx.attr.package

  recursive_deps = go_compile(ctx, pkg, ctx.files.srcs, archive, setupGOPATH=True)
  return struct(go_sources = ctx.files.srcs,
                go_archive = archive,
                go_recursive_deps = recursive_deps)

def go_binary_impl(ctx):
  gotool = ctx.file._go

  archive = ctx.new_file(ctx.configuration.bin_dir, ctx.label.name + ".a")
  go_compile(ctx, 'main', ctx.files.srcs, archive)

  recursive_deps = set()
  for t in ctx.attr.deps:
    recursive_deps += t.go_recursive_deps + [t.go_archive]

  link_binary(ctx, ctx.outputs.executable, archive, recursive_deps)
  return struct()

def go_test_impl(ctx):
  testmain_generator = ctx.file._go_testmain_generator

  lib = ctx.attr.library
  pkg = ctx.attr.go_package_prefix + lib.label.package

  test_srcs = ctx.files.srcs
  testmain = ctx.new_file(ctx.configuration.genfiles_dir, ctx.label.name + "main.go")
  cmd = (
      "set -e;" +
      testmain_generator.path + " " + pkg + " " + testmain.path + " " +
      cmd_helper.join_paths(" ", set(test_srcs)) + ";")
  ctx.action(
      inputs = test_srcs + [testmain_generator],
      outputs = [testmain],
      mnemonic = 'GoTestMain',
      command = cmd,
      use_default_shell_env = True)

  archive = ctx.new_file(ctx.configuration.bin_dir, ctx.label.name + ".a")
  go_compile(ctx, pkg, ctx.files.srcs + ctx.attr.library.go_sources, archive,
             extra_archives = ctx.attr.library.go_recursive_deps,
             setupGOPATH = True)

  test_archive = ctx.new_file(ctx.configuration.bin_dir, ctx.label.name + "main.a")
  go_compile(ctx, 'main', [testmain], test_archive, extra_archives=[archive])

  recursive_deps = lib.go_recursive_deps + [archive]
  for t in ctx.attr.deps:
    recursive_deps += t.go_recursive_deps + [t.go_archive]

  binary = ctx.outputs.executable
  link_binary(ctx, binary, test_archive, recursive_deps)

  runfiles = ctx.runfiles(files = [binary], collect_data = True)
  return struct(runfiles = runfiles)

base_attrs = {
    "srcs": attr.label_list(allow_files = FileType([".go"])),
    "deps": attr.label_list(
        allow_files = False,
        providers = [
            "go_archive",
        ],
    ),
    "go_build": attr.int(),
    # TODO(schroederc): put package prefix into common configuration file
    "go_package_prefix": attr.string(default = "github.com/google/shipshape/"),
    "_go": attr.label(
        default = Label("//tools/go"),
        allow_files = True,
        single_file = True,
    ),
    "_goroot": attr.label(
        default = Label("//tools/go:GOROOT"),
        allow_files = True,
        single_file = True,
    ),
}

go_library = rule(
    go_library_impl,
    attrs = base_attrs + {
        "package": attr.string(),
    },
    outputs = {"archive": "%{name}.a"},
)

go_binary = rule(
    go_binary_impl,
    attrs = base_attrs,
    executable = True,
)

go_test = rule(
    go_test_impl,
    attrs = base_attrs + {
        "data": attr.label_list(
            allow_files = True,
            cfg = DATA_CFG,
        ),
        "library": attr.label(providers = [
            "go_sources",
            "go_recursive_deps",
        ]),
        "_go_testmain_generator": attr.label(
            default = Label("//tools/go:testmain_generator"),
            single_file = True,
        ),
    },
    executable = True,
    test = True,
)

def go_package(name=None, package=None,
               srcs="", deps=[], test_deps=[], test_args=[], test_data=[],
               tests=True, exclude_srcs=[],
               visibility=None):
  if not name:
    name = PACKAGE_NAME.split("/")[-1]

  if srcs and not srcs.endswith("/"):
    srcs += "/"

  exclude = []
  for src in exclude_srcs:
    exclude += [srcs+src]

  lib_srcs, test_srcs = [], []
  for src in native.glob([srcs+"*.go"], exclude=exclude, exclude_directories=1):
    if src.endswith("_test.go"):
      test_srcs += [src]
    else:
      lib_srcs += [src]

  go_library(
    name = name,
    srcs = lib_srcs,
    deps = deps,
    package = package,
    visibility = visibility,
  )

  if tests and test_srcs:
    go_test(
      name = name + "_test",
      srcs = test_srcs,
      library = ":" + name,
      deps = test_deps,
      args = test_args,
      data = test_data,
      visibility = ["//visibility:private"],
    )
